// File:     series.h
// Author:   Jeff Miller
// Date:     12-17-97
// Abstract: Time series prediction class definitions

#include <assert.h>
#include "firnet.h"

#ifndef __SERIES_H__

class TimeSeries  // A time series, with prediction methods
{
public:

   TimeSeries();
   ~TimeSeries() { Deallocate(); }

   //
   // Time series access
   //

   double& operator[](int n) { return m_Vectors[n]; }
   int     Length() { return m_NumVectors; }
   void    BlankSeries(int Dimension, int NumVectors);
   void    PrintSeries();

   //
   // State I/O
   //

   int     LoadSeries(char* FileName);    // Read series from file
   int     SaveSeries(char* BaseName);    // Write series to file.out
   int     LoadNetwork(char* BaseName);   // Load network from file.net
   int     SaveNetwork(char* BaseName);   // Write network to file.net

   //
   // Prediction network methods
   //

   void    SetNetworkSize(int Size);

   void    SetHiddenNodes(int HiddenLayer, int NumNodes) {
      // HiddenLayer is between 0 and NetworkSize - 2
      m_Network->SetNodes(HiddenLayer + 1, NumNodes);
   }

   void    SetLayerTaps(int Layer, int NumTaps) {
      // Layer is between 0 and NetworkSize - 1
      m_Network->SetTaps(Layer, NumTaps);
   }

   void    SetTrainingParams(int Epochs, double Rate, int Freq, double Momentum, int Decrease) {
      m_Network->SetTrainingParams(Epochs, Rate, Freq, Momentum, Decrease);
   }

   void    RandomizeNetwork() {
      m_Network->RandomizeWeights();
   }

   void    TrainNetwork() {
      m_Network->Train(m_NumVectors - 1, m_Vectors, m_Vectors + 1);
   }

   void    Predict(int StartTime, int EndTime, TimeSeries& Output);

private:

   void    Deallocate();

   double* m_Vectors;		// The time series itself
   int     m_NumVectors;	// # of points in the series
   int     m_Dimension;		// Dimension of the input vectors

   FIRNet* m_Network;		// FIR neural network used in prediction
   int     m_NetworkSize;  // # of layers in network (2 or 3)
};

#define __SERIES_H__
#endif
